" Vim syntax file
" Language:	C
" Maintainer:	Bram Moolenaar <mool@oce.nl>
" Last change:	1997 April 20

" Remove any old syntax stuff hanging around
syn clear

" A bunch of useful C keywords
syn keyword cStatement	goto break return continue asm
syn keyword cLabel		case default
syn keyword cConditional	if else switch with
syn keyword cRepeat		while for do foreach

syn keyword cTodo contained	TODO FIXME XXX

" String and Character constants
" Highlight special characters (those which have a backslash) differently
syn match   cSpecial contained "\\[0-9][0-9][0-9]\|\\."
syn region  cString		  start=+"+  skip=+\\\\\|\\"+  end=+"+  contains=cSpecial
syn match   cCharacter	  "'[^\\]'"
syn match   cSpecialCharacter  "'\\.'"

"catch errors caused by wrong parenthesis
syn region	cParen		transparent start='(' end=')' contains=ALLBUT,cParenError,cIncluded,cSpecial,cTodo,cUserLabel
syn match	cParenError	")"
syn match 	cInParen	contained "[{}]"
hi link cParenError cError
hi link cInParen cError

"integer number, or floating point number without a dot and with "f".
syn case ignore
syn match  cNumber		"\<[0-9]\+\(u\=l\=\|lu\|f\)\>"
"floating point number, with dot, optional exponent
syn match  cFloat		"\<[0-9]\+\.[0-9]*\(e[-+]\=[0-9]\+\)\=[fl]\=\>"
"floating point number, starting with a dot, optional exponent
syn match  cFloat		"\.[0-9]\+\(e[-+]\=[0-9]\+\)\=[fl]\=\>"
"floating point number, without dot, with exponent
syn match  cFloat		"\<[0-9]\+e[-+]\=[0-9]\+[fl]\=\>"
"hex number
syn match  cNumber		"0x[0-9a-f]\+\(u\=l\=\|lu\)\>"
"syntax match  cIdentifier	"\<[a-z_][a-z0-9_]*\>"
syn case match

syn region cComment		start="/\*"  end="\*/" contains=cTodo
syn match  cComment		"//.*" contains=cTodo
syn match  cCommentError	"\*/"

syn keyword cOperator		sizeof
syn keyword cType			int long short char void size_t
syn keyword cType			signed unsigned float double
syn keyword cStructure		struct union enum typedef
syn keyword cStorageClass	static register auto volatile extern const

syn region cPreCondit	start="^[ \t]*#[ \t]*\(if\>\|ifdef\>\|ifndef\>\|elif\>\|else\>\|endif\>\)"  skip="\\$"  end="$" contains=cComment,cString,cCharacter,cNumber,cCommentError
syn region cIncluded contained start=+"+  skip=+\\\\\|\\"+  end=+"+
syn match  cIncluded contained "<[^>]*>"
syn match  cInclude		"^[ \t]*#[ \t]*include\>[ \t]*["<]" contains=cIncluded
"syntax match  cLineSkip		"\\$"
syn region cDefine		start="^[ \t]*#[ \t]*\(define\>\|undef\>\)" skip="\\$" end="$" contains=ALLBUT,cPreCondit,cIncluded,cInclude,cDefine,cInParen
syn region cPreProc		start="^[ \t]*#[ \t]*\(pragma\>\|line\>\|warning\>\|warn\>\|error\>\)" skip="\\$" end="$" contains=ALLBUT,cPreCondit,cIncluded,cInclude,cDefine,cInParen

syn sync ccomment cComment minlines=10

if !exists("did_c_syntax_inits")
  let did_c_syntax_inits = 1
  " The default methods for hiing.  Can be overridden later
  hi link cLabel		Label
  hi link cUserLabel		Label
  hi link cConditional	Conditional
  hi link cRepeat	Repeat
  hi link cCharacter	Character
  hi link cSpecialCharacter cSpecial
  hi link cNumber	Number
  hi link cFloat		Float
  hi link cCommentError	cError
  hi link cOperator	Operator
  hi link cStructure	Structure
  hi link cStorageClass	StorageClass
  hi link cInclude	Include
  hi link cPreProc	PreProc
  hi link cDefine	Macro
  hi link cIncluded	cString
  hi link cError		Error
  hi link cStatement	Statement
  hi link cPreCondit	PreCondit
  hi link cType		Type
  hi link cString	String
  hi link cComment	Comment
  hi link cSpecial	SpecialChar
  hi link cTodo		Todo
  "hi link cIdentifier	Identifier
endif

let b:current_syntax = "k"

" vim: ts=8
