import java.applet.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.awt.event.*;
import java.net.URL;


public class WorkCanvas extends Canvas implements Runnable {
   int frameNumber = -1;
   int delay;
   Thread animatorThread;
   boolean frozen = true;
   int squareSize = 5;
   boolean fillColumnTop = true;

   public WorkCanvas(int fps, int sqsize) {
      setSize(20,20);
      squareSize=sqsize;
      delay = (fps > 0) ? (1000 / fps) : 100;
      start();
   }

   public void start() {
       if (frozen) {
           //Do nothing.  
       } else {
           //Start animating!
           if (animatorThread == null) {
               animatorThread = new Thread(this);
           }
           animatorThread.start();
       }
   }

   public void stop() {
       if (animatorThread!=null){
          animatorThread=null;
       }
   }

   public void freeze(boolean freeze){
      frozen=freeze;
      if (frozen){
         stop();
      }
      else{
         start();
      }
   }

   public void run() {
       Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
       long startTime = System.currentTimeMillis();
       while (Thread.currentThread() == animatorThread) {
           frameNumber++;
           repaint();
           try {
               startTime += delay;
               Thread.sleep(Math.max(0,startTime-System.currentTimeMillis()));
           } catch (InterruptedException e) {
               break;
           }
       }
   }

   public void paint(Graphics g) {
       update(g);
   }

   public void update(Graphics g) {
       Color bg = getBackground();
       Color fg = getForeground();
       Dimension d = size();
       boolean fillSquare;
       boolean fillNextFrame;
       int rowWidth = 0;
       int x = 0, y = 0;
       int w, h;
       int tmp;
       fillSquare = fillColumnTop;
       fillColumnTop = !fillColumnTop;
       tmp = frameNumber % squareSize;
       if (tmp == 0) {
           w = squareSize;
           fillNextFrame = !fillSquare;
       } else {
           w = tmp;
           fillNextFrame = fillSquare;
       }
       while (x < d.width) {
          int colHeight = 0;
          while (y < d.height) {
             colHeight += squareSize;
             if (colHeight > d.height) {
                 h = d.height - y;
             } else {
                 h = squareSize;
             }
             if (fillSquare) {
                 g.fillRect(x, y, w, h);
                 fillSquare = false;
             } else {
                 g.setColor(bg);
                 g.fillRect(x, y, w, h);
                 g.setColor(fg);
                 fillSquare = true;
             } 
             y += h;
          } //while y
          x += w;
          y = 0;
          w = squareSize;
          rowWidth += w;
          if (rowWidth > d.width) {
              w = d.width - x;
          }
          fillSquare = fillColumnTop;
          fillColumnTop = !fillColumnTop;
       } //while x
       fillColumnTop = fillNextFrame;
   }
}
